from django.conf.urls import url
from .apis import CommentListView, CommentDetailView

urlpatterns = [
    url(r'^list/?$', CommentListView.as_view(), name='lists'),
    url(r'^list/(?P<pk>\d+)/?$', CommentDetailView.as_view(), name='detail'),
]



from django.shortcuts import render
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import serializers
from .models import *
from rest_framework.permissions import IsAuthenticated, AllowAny
from rest_framework.authentication import TokenAuthentication
from django.db.models import Q
from django.db.models import F
import json

from django.http import JsonResponse


class CommentAPISerializer(serializers.ModelSerializer):
    class Meta:
        model = Comment
        fields = '__all__'


class CommentAPIView(APIView):
    permission_classes = (AllowAny,)
    params = None

    def get(self, request, format=None):
        self.params = request.query_params  # 返回QueryDict类型

        page = int(self.params['page']) - 1 if 'page' in self.params else 0
        size = int(self.params['size']) if 'size' in self.params else 10

        print(">>>page", page)
        print(">>>size", size)

        comments = Comment.objects.all()\
            # .filter(Q(account=self.params['uid']) | Q(their_account=self.params['id'])).order_by('id')
        total = comments.count()
        now_pages = comments[page * size:page * size + size] if page >= 0 else comments
        data = now_pages.annotate(
            username = F('user__username'),
        ).values(
            'id',
            'thread',
            'user',
            'create_time',
            'message',
            'by_comment_id',
            'weight',
            'likes',
            'comments',
            'favorite',
            'shares',
        )

        return Response({
            'err': 0,
            'msg': 'OK',
            'data': {'total': total, 'list': data, },
            'request': self.params,
            # 'serializer': serializer.data,
        })

    def post(self, request):
        self.params = request.query_params

        return Response({
            'err': 0,
            'msg': 'OK',
            'data': {},
            'request': self.params,
        })


from django.contrib import admin
from .models import *


class CommentAdmin(admin.ModelAdmin):
    list_display = ('id', 'is_deleted', 'thread_id', 'user_id', 'ip', 'message', 'commented_id',
                    'weight', 'likes', 'favorite', 'comments', 'shares')
    search_fields = ('id', 'is_deleted', 'thread_id', 'user_id', 'ip', 'message', 'commented_id',
                     'weight', 'likes', 'favorite', 'comments', 'shares')
    fields = ('is_deleted', 'thread_id', 'user_id', 'ip', 'message', 'commented_id',
              'weight', 'likes', 'favorite', 'comments', 'shares')


# Register your models here.
admin.site.register(Comment, CommentAdmin)
admin.site.register(Comment1, CommentAdmin)
admin.site.register(Comment2, CommentAdmin)
admin.site.register(Comment3, CommentAdmin)
admin.site.register(Comment4, CommentAdmin)
admin.site.register(Comment5, CommentAdmin)
admin.site.register(Comment6, CommentAdmin)
admin.site.register(Comment7, CommentAdmin)
admin.site.register(Comment8, CommentAdmin)
admin.site.register(Comment9, CommentAdmin)



"""
Created on 2022-05-01
@author:刘飞
@description:评论模块逻辑处理
"""
from django.db.models import F
from rest_framework import serializers
from .models import CommentAbstract
from xj_thread.models import ThreadStatistic
from xj_user.models import BaseInfo, DetailInfo


# 从数据库获取评论树
def get_comment_tree(comment, obj):
    tree = []
    for c in obj:
        user_obj = BaseInfo.objects.filter(id=c.user_id).first()
        user_detail_obj = DetailInfo.objects.filter(user_id=c.user_id).first()
        menu_data = {
            "id": c.id,
            "avatar": user_detail_obj.avatar if user_detail_obj else None,
            "nickname": user_detail_obj.nickname if user_detail_obj else None,
            "is_deleted": c.is_deleted,
            "user_id": c.user_id,
            "full_name": user_obj.full_name if user_obj else None,
            "user_name": user_obj.user_name if user_obj else None,
            "ip": c.ip,
            # "create_time": dateutil.parser.parse(c.create_time).strftime('%Y-%m-%d %H:%M:%S'),
            "create_time": c.create_time.strftime('%Y-%m-%d %H:%M:%S'),
            "message": c.message,
            "weight": c.weight,
            "likes": c.likes,
            "favorite": c.favorite,
            "comments": c.comments,
            "shares": c.shares,
            "children": []
        }
        child = comment.objects.filter(commented_id=c.id, is_deleted=False)
        if child:
            menu_data["children"].append(get_comment_tree(comment, child))
        tree.append(menu_data)
    return tree


class CommentServices:
    def __init__(self):
        pass

    @staticmethod
    def comment_list_read(request):
        """评论列表"""
        if request.method == 'GET':
            thread_id = request.query_params.get('thread_id')
            if not thread_id:
                raise serializers.ValidationError('信息id未传。')
            comment = CommentAbstract.get_student_db_model(thread_id=thread_id)
            obj = comment.objects.filter(thread_id=thread_id, commented_id=None, is_deleted=False)
            res = get_comment_tree(comment, obj)
            return res, None

    @staticmethod
    def comment_list_create(request):
        """评论表新增"""
        thread_id = request.data.get('thread_id')
        message = request.data.get('message')
        commented_id = request.data.get('commented_id')
        if not thread_id:
            raise serializers.ValidationError('信息id未传。')

        # 信息统计表更新数据
        ThreadStatistic.objects.filter(thread_id=thread_id).update(comments=F('comments') + 1)

        # 找到对应评论表
        comment = CommentAbstract.get_student_db_model(thread_id=thread_id)
        # 父评论信息更新
        if commented_id:
            comment.objects.filter(id=commented_id).update(comments=F('comments') + 1)
        # 评论新增
        obj = comment()
        obj.user_id = request.user.get('user_id', None)
        obj.thread_id_id = thread_id
        obj.message = message
        obj.commented_id = commented_id
        obj.save()
        return None, None

    @staticmethod
    def comment_detail(request, pk):
        thread_id = request.data.get('thread_id')
        if not thread_id:
            raise serializers.ValidationError('信息id未传。')
        comment = CommentAbstract.get_student_db_model(thread_id=thread_id)  # 对应的数据表
        comment_obj = comment.objects.filter(id=pk).first()  # 这条评论

        if request.method == 'DELETE':
            if comment_obj:
                # 信息统计表数据更新
                ThreadStatistic.objects.filter(thread_id=thread_id).update(comments=F('comments') - 1)
                # 父评论更新数据
                if comment_obj.commented_id:
                    comment.objects.filter(id=comment_obj.commented_id).update(comments=F('comments') - 1)

                comment_obj.is_deleted = True
                comment_obj.save()
            return None, None
